@@ -54,6 +54,12 @@ describe Rack::Multipart do
     params["text/plain; charset=US-ASCII"].must_equal ["contents"]
   end
 
+  deprecated "parses multipart content when called using Rack::Request#parse_multipart" do
+    request = Rack::Request.new(Rack::MockRequest.env_for("/", multipart_fixture(:content_type_and_no_disposition)))
+    params = request.send(:parse_multipart)
+    params["text/plain; charset=US-ASCII"].must_equal ["contents"]
+  end
+
   it "parses multipart content when content type is present but disposition is not when using IO" do
     read, write = IO.pipe
     env = multipart_fixture(:content_type_and_no_disposition)
@@ -197,7 +203,7 @@ describe Rack::Multipart do
     env = Rack::MockRequest.env_for '/', fixture
     lambda {
       Rack::Multipart.parse_multipart(env)
-    }.must_raise Rack::Multipart::Error
+    }.must_raise Rack::Multipart::EmptyContentError
     rd.close
 
     err = thr.value
@@ -205,239 +211,6 @@ describe Rack::Multipart do
     wr.close
   end
 
-  it "rejects excessive data before boundary" do
-    rd, wr = IO.pipe
-    def rd.rewind; end
-    wr.sync = true
-
-    thr = Thread.new do
-      begin
-        longer = "0123456789" * 1024 * 1024
-        (1024 * 1024).times do
-           wr.write(longer)
-        end
-
-        wr.write("\r\n\r\n--AaB03x")
-        wr.write("\r\n")
-        wr.write('content-disposition: form-data; name="a"; filename="a.txt"')
-        wr.write("\r\n")
-        wr.write("content-type: text/plain\r\n")
-        wr.write("\r\na")
-        wr.write("--AaB03x--\r\n")
-        wr.close
-      rescue => err # this is EPIPE if Rack shuts us down
-        err
-      end
-    end
-
-    fixture = {
-      "CONTENT_TYPE" => "multipart/form-data; boundary=AaB03x",
-      "CONTENT_LENGTH" => (1024 * 1024 * 8).to_s,
-      :input => rd,
-    }
-
-    env = Rack::MockRequest.env_for '/', fixture
-    lambda {
-      Rack::Multipart.parse_multipart(env)
-    }.must_raise(Rack::Multipart::Error).message.must_equal "multipart boundary not found within limit"
-    rd.close
-
-    err = thr.value
-    err.must_be_instance_of Errno::EPIPE
-    wr.close
-  end
-
-  it "rejects excessive mime header size" do
-    rd, wr = IO.pipe
-    def rd.rewind; end
-    wr.sync = true
-
-    thr = Thread.new do
-      begin
-        wr.write("\r\n\r\n--AaB03x")
-        wr.write("\r\n")
-        wr.write('content-disposition: form-data; name="a"; filename="a.txt"')
-        wr.write("\r\n")
-        wr.write("content-type: text/plain\r\n")
-        longer = "0123456789" * 1024 * 1024
-        (1024 * 1024).times do
-          wr.write(longer)
-        end
-        wr.write("\r\na")
-        wr.write("--AaB03x--\r\n")
-        wr.close
-      rescue => err # this is EPIPE if Rack shuts us down
-        err
-      end
-    end
-
-    fixture = {
-      "CONTENT_TYPE" => "multipart/form-data; boundary=AaB03x",
-      "CONTENT_LENGTH" => (1024 * 1024 * 8).to_s,
-      :input => rd,
-    }
-
-    env = Rack::MockRequest.env_for '/', fixture
-    lambda {
-      Rack::Multipart.parse_multipart(env)
-    }.must_raise(Rack::Multipart::Error).message.must_equal "multipart mime part header too large"
-    rd.close
-
-    err = thr.value
-    err.must_be_instance_of Errno::EPIPE
-    wr.close
-  end
-
-  it "parses when the MIME head terminator straddles the BUFSIZE boundary" do
-    boundary = '------WebKitFormBoundaryysVLFAjttLkewYBx'
-
-    data = StringIO.new
-    data.write("--#{boundary}")
-    data.write("\r\n")
-
-    data.write('content-disposition: form-data; name="a"')
-    data.write("\r\n")
-    data.write("\r\n")
-    # Fill to the end of the first 1MB chunk so the header's `\r\n` is in the next chunk.
-    data.write("0" * (1024 * 1024 - 174))
-    data.write("\r\n")
-    data.write("--#{boundary}")
-    data.write("\r\n")
-    data.write('content-disposition: form-data; name="b"')
-    # First 1MB chunk separator is here
-    data.write("\r\n")
-    data.write("\r\n")
-    data.write("0" * (1024 * 1024 - 88))
-    data.write("\r\n")
-    data.write("--#{boundary}")
-    data.write("\r\n")
-    data.write('content-disposition: form-data; name="c"')
-    # Second 1MB chunk separator is here
-    data.write("\r\n")
-    data.write("\r\n")
-    data.write("hello")
-    data.write("\r\n")
-    data.write("--#{boundary}--\r\n")
-    data.rewind
-
-    fixture = {
-      "CONTENT_TYPE" => "multipart/form-data; boundary=#{boundary}",
-      "CONTENT_LENGTH" => data.length.to_s,
-      :input => data,
-    }
-
-    env = Rack::MockRequest.env_for '/', fixture
-    Rack::Multipart.parse_multipart(env).keys.must_equal(["a", "b", "c"])
-  end
-
-  it "rejects excessive buffered mime data size in a single parameter" do
-    rd, wr = IO.pipe
-    def rd.rewind; end
-    wr.sync = true
-
-    thr = Thread.new do
-      wr.write("--AaB03x")
-      wr.write("\r\n")
-      wr.write('content-disposition: form-data; name="a"')
-      wr.write("\r\n")
-      wr.write("content-type: text/plain\r\n")
-      wr.write("\r\n")
-      wr.write("0" * 17 * 1024 * 1024)
-      wr.write("--AaB03x--\r\n")
-      wr.close
-      true
-    rescue Errno::EPIPE
-      # Expected when the reader closes due to size limit violation
-      true
-    end
-
-    fixture = {
-      "CONTENT_TYPE" => "multipart/form-data; boundary=AaB03x",
-      "CONTENT_LENGTH" => (18 * 1024 * 1024).to_s,
-      :input => rd,
-    }
-
-    env = Rack::MockRequest.env_for '/', fixture
-    lambda {
-      Rack::Multipart.parse_multipart(env)
-    }.must_raise(Rack::Multipart::Error).message.must_equal "multipart data over retained size limit"
-    rd.close
-
-    thr.value.must_equal true
-    wr.close
-  end
-
-  it "rejects excessive buffered mime data size when split into multiple parameters" do
-    rd, wr = IO.pipe
-    def rd.rewind; end
-    wr.sync = true
-
-    thr = Thread.new do
-      4.times do |i|
-        wr.write("\r\n--AaB03x")
-        wr.write("\r\n")
-        wr.write("content-disposition: form-data; name=\"a#{i}\"")
-        wr.write("\r\n")
-        wr.write("content-type: text/plain\r\n")
-        wr.write("\r\n")
-        wr.write("0" * 4 * 1024 * 1024)
-      end
-      wr.write("\r\n--AaB03x--\r\n")
-      wr.close
-      true
-    rescue Errno::EPIPE
-      # Expected when the reader closes due to size limit violation
-      true
-    end
-
-    fixture = {
-      "CONTENT_TYPE" => "multipart/form-data; boundary=AaB03x",
-      "CONTENT_LENGTH" => (17 * 1024 * 1024).to_s,
-      :input => rd,
-    }
-
-    env = Rack::MockRequest.env_for '/', fixture
-    lambda {
-  p    Rack::Multipart.parse_multipart(env).keys
-    }.must_raise(Rack::Multipart::Error).message.must_equal "multipart data over retained size limit"
-    rd.close
-
-    thr.value.must_equal true
-    wr.close
-  end
-
-  it "allows large nonbuffered mime parameters" do
-    rd, wr = IO.pipe
-    def rd.rewind; end
-    wr.sync = true
-
-    thr = Thread.new do
-      wr.write("\r\n\r\n--AaB03x")
-      wr.write("\r\n")
-      wr.write('content-disposition: form-data; name="a"; filename="a.txt"')
-      wr.write("\r\n")
-      wr.write("content-type: text/plain\r\n")
-      wr.write("\r\n")
-      wr.write("0" * 16 * 1024 * 1024)
-      wr.write("\r\n--AaB03x--\r\n")
-      wr.close
-      true
-    end
-
-    fixture = {
-      "CONTENT_TYPE" => "multipart/form-data; boundary=AaB03x",
-      "CONTENT_LENGTH" => (17 * 1024 * 1024).to_s,
-      :input => rd,
-    }
-
-    env = Rack::MockRequest.env_for '/', fixture
-    Rack::Multipart.parse_multipart(env)['a'][:tempfile].read.bytesize.must_equal(16 * 1024 * 1024)
-    rd.close
-
-    thr.value.must_equal true
-    wr.close
-  end
-
   # see https://github.com/rack/rack/pull/1309
   it "parses strange multipart pdf" do
     boundary = '---------------------------932620571087722842402766118'
@@ -971,7 +744,7 @@ content-type: image/jpeg\r
     input = File.read(multipart_file("bad_robots"))
 
     req = Rack::Request.new Rack::MockRequest.env_for("/",
-                      "CONTENT_TYPE" => "multipart/form-data, boundary=1yy3laWhgX31qpiHinh67wJXqKalukEUTvqTzmon",
+                      "CONTENT_TYPE" => "multipart/form-data; boundary=1yy3laWhgX31qpiHinh67wJXqKalukEUTvqTzmon",
                       "CONTENT_LENGTH" => input.size,
                       :input => input)
 
@@ -1035,6 +808,21 @@ content-type: image/jpeg\r
     end
   end
 
+  it "treats a multipart limit of 0 as no limit" do
+    begin
+      previous_limit = Rack::Utils.multipart_total_part_limit
+      Rack::Utils.multipart_total_part_limit = 0
+
+      env = Rack::MockRequest.env_for '/', multipart_fixture(:three_files_three_fields)
+      params = Rack::Multipart.parse_multipart(env)
+      params['reply'].must_equal 'yes'
+      params['to'].must_equal 'people'
+      params['from'].must_equal 'others'
+    ensure
+      Rack::Utils.multipart_total_part_limit = previous_limit
+    end
+  end
+
   it "reaches a multipart file limit" do
     begin
       previous_limit = Rack::Utils.multipart_part_limit
@@ -1262,4 +1050,11 @@ content-type: image/png\r
     f.write(params["image/png"][0])
     f.length.must_equal 26473
   end
+
+  it "supports ISO-2022-JP-encoded part" do
+    env = Rack::MockRequest.env_for("/", multipart_fixture(:multiple_encodings))
+    params = Rack::Multipart.parse_multipart(env)
+    params["us-ascii"].must_equal("Alice")
+    params["iso-2022-jp"].must_equal("アリス")
+  end
 end
